
module i2c_cont (
    input               reset_n,    // reset
    input               clk,        // clock
    input       [2:0]   addr,       // internal address
    input               sel,        // internal select
    input               write,      // internal write enable
    input       [31:0]  wdata,      // internal write data
    output      [31:0]  rdata,      // internal read data
    output              waitrequest_n,  // internal waitrequest
    output              irq,        // internal interrupt
    output              scl_out,    // i2c clock output
    input               scl_in,     // i2c clock input
    output              scl_oe,     // i2c clock output enable
    output              sda_out,    // i2c data  output
    input               sda_in,     // i2c data  input
    output              sda_oe      // i2c data  output enable
);

//-------------- internal signals --------------
    wire                scl_oen;
    wire                sda_oen;

//-------------- logic description --------------
    assign rdata[31:8] = 24'b0 ;
    assign scl_oe = ~scl_oen;
    assign sda_oe = ~sda_oen;
   
    i2c_master_top i2c_master_top (
    	.wb_clk_i(clk),
        .wb_rst_i(1'b0),
        .arst_i(reset_n),
    	
    	.wb_adr_i(addr),
        .wb_dat_i(wdata[7:0]),
        .wb_dat_o(rdata[7:0]),
    	.wb_we_i(write),
        .wb_stb_i(sel),
        .wb_cyc_i(1'b1),
    	.wb_ack_o(waitrequest_n),
        .wb_inta_o(irq),
    	
    	.scl_pad_i(scl_in),
        .scl_pad_o(scl_out),
        .scl_padoen_o(scl_oen),
    	.sda_pad_i(sda_in),
        .sda_pad_o(sda_out),
        .sda_padoen_o(sda_oen)
    );

endmodule
