
module pinmux #(
    parameter           PA_WIDTH = 16,
    parameter           PB_WIDTH = 16,
    parameter           PC_WIDTH = 16,
    parameter           PD_WIDTH = 16,
    parameter           PE_WIDTH = 2
)(
    input                   reset_n,// reset
    input                   clk,    // clock
    input       [2:0]       addr,   // internal address
    input                   sel,    // internal select
    input                   write,  // internal write enable
    input       [31:0]      wdata,  // internal write data
    output    [31:0]      rdata,  // internal read data

    output  reg [PA_WIDTH-1:0]  pa_mux,  // pa multiplexer register
    output  reg [PB_WIDTH-1:0]  pb_mux,  // pb multiplexer register
    output  reg [PC_WIDTH-1:0]  pc_mux,  // pc multiplexer register
    output  reg [PD_WIDTH-1:0]  pd_mux,  // pd multiplexer register
    output  reg [PE_WIDTH-1:0]  pe_mux   // pe multiplexer register
);

//-------------- internal signals --------------
    reg         [15:0]  rdata_mux ;

//-------------- logic description --------------

//-------------- write data --------------
    always @(posedge clk or negedge reset_n) begin
        if(!reset_n) begin
            pa_mux <= 0;
            pb_mux <= 0;
            pc_mux <= 0;
            pd_mux <= 0;
            pe_mux <= 0;
        end
        else if(sel && write) begin
            case(addr)
                0 : pa_mux <= wdata[PA_WIDTH-1:0];
                1 : pb_mux <= wdata[PB_WIDTH-1:0];
                2 : pc_mux <= wdata[PC_WIDTH-1:0];
                3 : pd_mux <= wdata[PD_WIDTH-1:0];
                4 : pe_mux <= wdata[PE_WIDTH-1:0];
                default: ;
            endcase
        end
    end

//-------------- read data --------------
/*
    assign rdata = (addr == 2'b00)? pa_mux : 
                   (addr == 2'b01)? pb_mux :
                   (addr == 2'b10)? pc_mux :
                                    pd_mux ;
*/
	assign rdata = {16'b0, rdata_mux} ;


    always @(posedge clk or negedge reset_n) begin
        if(!reset_n)
            rdata_mux <= 0;
        else begin
            case(addr)
                0 : rdata_mux <= pa_mux;
                1 : rdata_mux <= pb_mux;
                2 : rdata_mux <= pc_mux;
                3 : rdata_mux <= {{(16-PD_WIDTH){1'b0}}, pd_mux} ;
                4 : rdata_mux <= {{(16-PE_WIDTH){1'b0}}, pe_mux} ;
                default: rdata_mux <= 0;
            endcase
        end
    end

endmodule
