
module pio_cont #(
    parameter           PA_WIDTH = 16,
    parameter           PB_WIDTH = 16,
    parameter           PC_WIDTH = 16,
    parameter           PD_WIDTH = 16,
    parameter           PE_WIDTH = 2
)(
    input                   reset_n,    // reset
    input                   clk,        // clock
    input       [3:0]       addr,       // internal address
    input                   sel,        // internal select
    input                   write,      // internal write enable
    input       [31:0]      wdata,      // internal write data
    output    [31:0]      rdata,      // internal read data
    output reg  [PA_WIDTH-1:0] pio_a_oe,   // pio_a output enable
    output reg  [PA_WIDTH-1:0] pio_a_out,  // pio_a output signals
    input       [PA_WIDTH-1:0] pio_a_in,   // pio_a input signals
    output reg  [PB_WIDTH-1:0] pio_b_oe,   // pio_b output enable
    output reg  [PB_WIDTH-1:0] pio_b_out,  // pio_b output signals
    input       [PB_WIDTH-1:0] pio_b_in,   // pio_b input signals
    output reg  [PC_WIDTH-1:0] pio_c_oe,   // pio_c output enable
    output reg  [PC_WIDTH-1:0] pio_c_out,  // pio_c output signals
    input       [PC_WIDTH-1:0] pio_c_in,   // pio_c input signals
    output reg  [PD_WIDTH-1:0] pio_d_oe,   // pio_d output enable
    output reg  [PD_WIDTH-1:0] pio_d_out,  // pio_d output signals
    input       [PD_WIDTH-1:0] pio_d_in,   // pio_d input signals
    output reg  [PE_WIDTH-1:0] pio_e_oe,   // pio_e output enable
    output reg  [PE_WIDTH-1:0] pio_e_out,  // pio_e output signals
    input       [PE_WIDTH-1:0] pio_e_in    // pio_e input signals
);
//-------------- internal signals --------------
    reg         [15:0]  rdata_mux ;

//-------------- logic description --------------

//-------------- register --------------
    always @(posedge clk or negedge reset_n) begin
        if(!reset_n) begin
            pio_a_out <= 0;
            pio_a_oe <= 0;
            pio_b_out <= 0;
            pio_b_oe <= 0;
            pio_c_out <= 0;
            pio_c_oe <= 0;
            pio_d_out <= 0;
            pio_d_oe <= 0;
            pio_e_out <= 0;
            pio_e_oe <= 0;
        end
        else if(sel && write) begin
            case(addr)
                0 : pio_a_out <= wdata[PA_WIDTH-1:0];
                1 : pio_a_oe  <= wdata[PA_WIDTH-1:0];
                2 : pio_b_out <= wdata[PB_WIDTH-1:0];
                3 : pio_b_oe  <= wdata[PB_WIDTH-1:0];
                4 : pio_c_out <= wdata[PC_WIDTH-1:0];
                5 : pio_c_oe  <= wdata[PC_WIDTH-1:0];
                6 : pio_d_out <= wdata[PD_WIDTH-1:0];
                7 : pio_d_oe  <= wdata[PD_WIDTH-1:0];
                8 : pio_e_out <= wdata[PE_WIDTH-1:0];
                9 : pio_e_oe  <= wdata[PE_WIDTH-1:0];
            endcase
        end
    end

//-------------- read data --------------
	assign rdata = {16'b0, rdata_mux} ;

    always @(posedge clk or negedge reset_n) begin
        if(!reset_n)
            rdata_mux <= 0;
        else begin
            case(addr)
                0 : rdata_mux <= pio_a_in;
                1 : rdata_mux <= pio_a_oe;
                2 : rdata_mux <= pio_b_in;
                3 : rdata_mux <= pio_b_oe;
                4 : rdata_mux <= pio_c_in;
                5 : rdata_mux <= pio_c_oe;
                6 : rdata_mux <= {{(16-PD_WIDTH){1'b0}}, pio_d_in} ;
                7 : rdata_mux <= {{(16-PD_WIDTH){1'b0}}, pio_d_oe} ;
                8 : rdata_mux <= {{(16-PE_WIDTH){1'b0}}, pio_e_in} ;
                9 : rdata_mux <= {{(16-PE_WIDTH){1'b0}}, pio_e_oe} ;
                default: rdata_mux <= 0;
            endcase
        end
    end

endmodule
