//`define DEBUG

module keim08_top(
    input           clk,        //50MHz
    input           reset_n,
    
    //Memory I/F
//	output	[22:0]	mem_a,
	output	[21:0]	mem_a,
	inout	[15:0]	mem_dq,
	output	        sdram_clk,
	output	        sdram_ras_n,
	output          sdram_we_n,
	output          sdram_cs_n,
	output	[ 1:0]  sdram_ba,
	output	[ 1:0]  sdram_dqm,
	output          sdram_cas_n,
	output          sdram_cke,
    output	        flash_ce_n,
    output	        flash_we_n,
    output	        flash_oe_n,
    output          flash_rst_n,
  
    //General Port
    inout   [15:0]  pa_pin,
    inout   [15:0]  pb_pin,
    inout   [15:0]  pc_pin,
    inout   [15:0]  pd_pin,
    inout   [ 1:0]  pe_pin
    
);
    
//---------------------- internal signals ----------------------
    wire            pll_0_locked;
    wire            ins_reset_n;
    
    //External I/F
    wire    [2:0]   spi_miso, spi_mosi, spi_sclk;
    wire    [1:0]   uart_rxd, uart_txd, uart_cts_n, uart_rts_n;
    wire            pwm_0;
    wire            pwm_1;
    //wire            pwm_2;
    wire    [7:0]   eint;
    wire            i2c_scl_oe, i2c_scl_out, i2c_scl_in;
    wire            i2c_sda_oe, i2c_sda_out, i2c_sda_in;
    
    wire    [15:0]  pa_mux, pa_oe, pa_out,
                    pio_a_oe, pio_a_out, pio_a_in,
                    func_a_oe, func_a_out, func_a_in;
    
    wire	[15:0]	pb_mux,  pb_oe, pb_out,
                    pio_b_oe, pio_b_out, pio_b_in,
                    func_b_oe, func_b_out, func_b_in;
                    
    wire	[15:0]	pc_mux, pc_oe, pc_out,
                    pio_c_oe, pio_c_out, pio_c_in,
                    func_c_oe, func_c_out;
//                    func_c_oe, func_c_out, func_c_in;
    
    wire	[15:0]	pd_mux, pd_oe, pd_out,
                    pio_d_oe, pio_d_out, pio_d_in,
                    func_d_oe, func_d_out, func_d_in;
    
    wire	[1:0]	pe_mux, pe_oe, pe_out,
                    pio_e_oe, pio_e_out, pio_e_in,
                    func_e_oe, func_e_out, func_e_in;
    
//---------------------- logic description ----------------------
    
//---------------------- generate delay reset ----------------------

//---------------------- instanse reset controller ----------------------
    reset_cont reset_cont(
        .clk            (clk),
        .reset_n        (reset_n),
        .pll_locked     (pll_0_locked),
        .ins_reset_n    (ins_reset_n)
    );

    //assign flash_rst_n = ins_reset_n;
    
//---------------------- core instanse ----------------------
    keim08core u0(
        .clk_clk                            (clk),
        .reset_reset_n                      (ins_reset_n),
        .altpll_areset_export               (1'b0),
        .altpll_locked_export               (pll_0_locked),
        .altpll_phasedone_export            (),
        .altpll_c1_clk                      (sdram_clk),
        
        .emif_flash_tcm_reset_n_out         (flash_rst_n),
        .emif_sdram_sdram_cke               (sdram_cke),
        .emif_sdram_sdram_cas_n             (sdram_cas_n),
        .emif_addr                          (mem_a),
        .emif_dq                            (mem_dq),
        .emif_sdram_sdram_we_n              (sdram_we_n),
        .emif_sdram_sdram_cs_n              (sdram_cs_n),
        .emif_flash_tcm_chipselect_n_out    (flash_ce_n),
        .emif_sdram_sdram_ba                (sdram_ba),
        .emif_sdram_sdram_ras_n             (sdram_ras_n),
        .emif_sdram_sdram_dqm               (sdram_dqm),
        .emif_flash_tcm_write_n_out         (flash_we_n),
        .emif_flash_tcm_read_n_out          (flash_oe_n),
        
		.spi_0_sclk                         (spi_sclk[0]),
		.spi_0_mosi                         (spi_mosi[0]),
		.spi_0_miso                         (spi_miso[0]),
		.spi_1_sclk                         (spi_sclk[1]),
		.spi_1_mosi                         (spi_mosi[1]),
		.spi_1_miso                         (spi_miso[1]),
		.spi_2_sclk                         (spi_sclk[2]),
		.spi_2_mosi                         (spi_mosi[2]),
		.spi_2_miso                         (spi_miso[2]),
        
        .uart_0_rxd                         (uart_rxd[0]),
        .uart_0_txd                         (uart_txd[0]),
        .uart_0_cts_n                       (uart_cts_n[0]),
        .uart_0_rts_n                       (uart_rts_n[0]),
        
        .uart_1_rxd                         (uart_rxd[1]),
        .uart_1_txd                         (uart_txd[1]),
        .uart_1_cts_n                       (uart_cts_n[1]),
        .uart_1_rts_n                       (uart_rts_n[1]),
        
        //.i2c_0_scl                          (i2c_scl),
        //.i2c_0_sda                          (i2c_sda),
        .i2c_0_scl_out                      (i2c_scl_out),
        .i2c_0_scl_in                       (i2c_scl_in),
        .i2c_0_scl_oe                       (i2c_scl_oe),
        .i2c_0_sda_out                      (i2c_sda_out),
        .i2c_0_sda_in                       (i2c_sda_in),
        .i2c_0_sda_oe                       (i2c_sda_oe),	
        
        .pwm_0_pwm_out                      (pwm_0),
        .pwm_1_pwm_out                      (pwm_1),
        //.pwm_2_pwm_out                      (pwm_2),
        
		.pio_pio_a_oe                       (pio_a_oe),
		.pio_pio_a_out                      (pio_a_out),
		.pio_pio_a_in                       (pio_a_in),
		.pio_pio_b_oe                       (pio_b_oe),
		.pio_pio_b_out                      (pio_b_out),
		.pio_pio_b_in                       (pio_b_in),
		.pio_pio_c_oe                       (pio_c_oe),
		.pio_pio_c_out                      (pio_c_out),
		.pio_pio_c_in                       (pio_c_in),
		.pio_pio_d_oe                       (pio_d_oe),
		.pio_pio_d_out                      (pio_d_out),
		.pio_pio_d_in                       (pio_d_in),
		.pio_pio_e_oe                       (pio_e_oe),
		.pio_pio_e_out                      (pio_e_out),
		.pio_pio_e_in                       (pio_e_in),
        
        .pinmux_pa_mux                      (pa_mux),
        .pinmux_pb_mux                      (pb_mux),
        .pinmux_pc_mux                      (pc_mux),
        .pinmux_pd_mux                      (pd_mux),
        .pinmux_pe_mux                      (pe_mux),
        
		.int_int                            (eint)
    );
    
//---------------------- port a function ----------------------
    assign {func_a_oe[ 0], func_a_out[ 0]} = {1'b1, uart_txd[0]};
    assign {func_a_oe[ 1], func_a_out[ 1]} = {1'b0, 1'b0};
    assign {func_a_oe[ 2], func_a_out[ 2]} = {1'b1, uart_rts_n[0]};
    assign {func_a_oe[ 3], func_a_out[ 3]} = {1'b0, 1'b0};
    assign {func_a_oe[ 4], func_a_out[ 4]} = {1'b1, uart_txd[1]};
    assign {func_a_oe[ 5], func_a_out[ 5]} = {1'b0, 1'b0};
    assign {func_a_oe[ 6], func_a_out[ 6]} = {1'b1, uart_rts_n[1]};
    assign {func_a_oe[ 7], func_a_out[ 7]} = {1'b0, 1'b0};
    assign {func_a_oe[ 8], func_a_out[ 8]} = {1'b1, spi_sclk[0]};
    assign {func_a_oe[ 9], func_a_out[ 9]} = {1'b0, 1'b0};
    assign {func_a_oe[10], func_a_out[10]} = {1'b1, spi_mosi[0]};
    assign {func_a_oe[11], func_a_out[11]} = {1'b0, 1'b0};
    assign {func_a_oe[12], func_a_out[12]} = {1'b1, spi_sclk[1]};
    assign {func_a_oe[13], func_a_out[13]} = {1'b0, 1'b0};
    assign {func_a_oe[14], func_a_out[14]} = {1'b1, spi_mosi[1]};
    assign {func_a_oe[15], func_a_out[15]} = {1'b0, 1'b0};
                        
    assign uart_rxd[0] = func_a_in[1];
    assign uart_cts_n[0] = func_a_in[3];
    assign uart_rxd[1] = func_a_in[5];
    assign uart_cts_n[1] = func_a_in[7];
    assign spi_miso[0] = func_a_in[11];
    assign spi_miso[1] = func_a_in[15];

//---------------------- port b function ----------------------
    assign {func_b_oe[ 0], func_b_out[ 0]} = {1'b1, spi_sclk[2]};
    assign {func_b_oe[ 1], func_b_out[ 1]} = {1'b0, 1'b0};
    assign {func_b_oe[ 2], func_b_out[ 2]} = {1'b1, spi_mosi[2]};
    assign {func_b_oe[ 3], func_b_out[ 3]} = {1'b0, 1'b0};
    assign {func_b_oe[ 4], func_b_out[ 4]} = {1'b0, 1'b0};
    assign {func_b_oe[ 5], func_b_out[ 5]} = {1'b0, 1'b0};
    assign {func_b_oe[ 6], func_b_out[ 6]} = {1'b0, 1'b0};
    assign {func_b_oe[ 7], func_b_out[ 7]} = {1'b0, 1'b0};
    assign {func_b_oe[ 8], func_b_out[ 8]} = {1'b1, pwm_0};
    assign {func_b_oe[ 9], func_b_out[ 9]} = {1'b1, pwm_1};
    assign {func_b_oe[10], func_b_out[10]} = {1'b0, 1'b0};
    assign {func_b_oe[11], func_b_out[11]} = {1'b0, 1'b0};
    assign {func_b_oe[12], func_b_out[12]} = {1'b0, 1'b0};
    assign {func_b_oe[13], func_b_out[13]} = {1'b0, 1'b0};
    assign {func_b_oe[14], func_b_out[14]} = {1'b0, 1'b0};
    assign {func_b_oe[15], func_b_out[15]} = {1'b0, 1'b0};
                        
    assign spi_miso[2] = func_b_in[3];
                        
//---------------------- port c function ----------------------
    assign func_c_out =  16'b0;
    assign func_c_oe  =  16'b0;
 
  `ifdef DEBUG
    assign pc_pin[0] = flash_ce_n;
    assign pc_pin[1] = flash_we_n;
    assign pc_pin[2] = flash_oe_n;
    assign pc_pin[4] = mem_dq[0];
    assign pc_pin[5] = mem_dq[1];
    assign pc_pin[6] = mem_a[0];
    assign pc_pin[7] = mem_a[1];
    assign pc_pin[8] = sdram_cs_n;
    assign pc_pin[9] = sdram_ras_n;
    assign pc_pin[10] = sdram_cas_n;
    assign pc_pin[11] = sdram_we_n;
  `endif
 
//---------------------- port d function ----------------------
    assign func_d_out =  16'b0;
    assign func_d_oe  =  16'b0;
    
    assign eint = func_d_in[7:0];

//---------------------- port e function ----------------------
    assign {func_e_oe[ 0], func_e_out[ 0]} = {i2c_scl_oe, i2c_scl_out};
    assign {func_e_oe[ 1], func_e_out[ 1]} = {i2c_sda_oe, i2c_sda_out};

    assign i2c_scl_in = func_e_in[0];
    assign i2c_sda_in = func_e_in[1];
    
//---------------------- bus multiplex ----------------------
    genvar i;
    generate
    for(i = 0; i < 16; i = i + 1) begin : gen_pa
        assign func_a_in[i] = (pa_mux[i])? pa_pin[i] : 1'b0;
        assign pio_a_in[i] = (pa_mux[i])? 1'b0 : pa_pin[i];
        assign pa_out[i] = (pa_mux[i])? func_a_out[i] : pio_a_out[i];
        assign pa_oe[i]  = (pa_mux[i])? func_a_oe[i]  : pio_a_oe[i];
        assign pa_pin[i] = (pa_oe[i])? pa_out[i] : 1'bz;
    end
    
    for(i = 0; i < 16; i = i + 1) begin : gen_pb
        assign func_b_in[i] = (pa_mux[i])? pb_pin[i] : 1'b0;
        assign pio_b_in[i] = (pb_mux[i])? 1'b0 : pb_pin[i];
        assign pb_out[i] = (pb_mux[i])? func_b_out[i] : pio_b_out[i];
        assign pb_oe[i]  = (pb_mux[i])? func_b_oe[i]  : pio_b_oe[i];
        assign pb_pin[i] = (pb_oe[i])? pb_out[i] : 1'bz;
    end

    for(i = 0; i < 16; i = i + 1) begin : gen_pc
//        assign func_c_in[i] = (pc_mux[i])? pc_pin[i] : 1'b0;
        assign pio_c_in[i] = (pc_mux[i])? 1'b0 : pc_pin[i];
        assign pc_out[i] = (pc_mux[i])? func_c_out[i] : pio_c_out[i];
        assign pc_oe[i]  = (pc_mux[i])? func_c_oe[i]  : pio_c_oe[i];
      `ifdef DEBUG
      `else
        assign pc_pin[i] = (pc_oe[i])? pc_out[i] : 1'bz;
      `endif
    end
    
    for(i = 0; i < 16; i = i + 1) begin : gen_pd
        assign func_d_in[i] = (pd_mux[i])? pd_pin[i] : 1'b0;
        assign pio_d_in[i] = (pd_mux[i])? 1'b0 : pd_pin[i];
        assign pd_out[i] = (pd_mux[i])? func_d_out[i] : pio_d_out[i];
        assign pd_oe[i]  = (pd_mux[i])? func_d_oe[i]  : pio_d_oe[i];
        assign pd_pin[i] = (pd_oe[i])? pd_out[i] : 1'bz;
    end

    for(i = 0; i < 2; i = i + 1) begin : gen_pe
        assign func_e_in[i] = (pe_mux[i])? pe_pin[i] : 1'b0;
        assign pio_e_in[i] = (pe_mux[i])? 1'b0 : pe_pin[i];
        assign pe_out[i] = (pe_mux[i])? func_e_out[i] : pio_e_out[i];
        assign pe_oe[i]  = (pe_mux[i])? func_e_oe[i]  : pio_e_oe[i];
        assign pe_pin[i] = (pe_oe[i])? pe_out[i] : 1'bz;
    end
    
    endgenerate

endmodule


 
